@@ -1,3 +1,5 @@
+from contexture import __version__
+
 import json
 import logging
 import os
@@ -22,35 +24,62 @@ def faux_record(obj):
     return faux
 
 
-# def publish(obj, headers=None, exchange='default', routing_key='lc-handler'):
-#     'Shorthand to publish anything anywhere'
+def qitems(queue):
+    while True:
+        try:
+            yield queue.get_nowait()
+        except Empty:
+            raise StopIteration
 
 
 # TODO: add heartbeat?
 class AMQPHandler(logging.Handler):
 
     INTERVAL = 1
-    MAXQUEUE = 300
 
-    def __init__(self, url, exchange='lc-topic', exchange_type='topic', headers={}):
+    # singleton stuff
+    _thread = None
+    _queue = None
+    _channel = None
+    _connection = None
+
+    def __init__(self, url,
+                 exchange='lc-topic',
+                 exchange_type='topic',
+                 headers={},
+                 singleton=True,
+                 maxqueue=300,
+                 reconnect_wait=10):
         self._url = url
         self._exchange = exchange
         self._headers = headers
         self._type = exchange_type
-        self._queue = Queue(self.MAXQUEUE)
         self._running = True
         self._guid = str(uuid.uuid4())
         env = dict(host=socket.gethostname(),
                    pid=os.getpid(),
                    argv=sys.argv,
+                   tid=threading.current_thread().ident,
+                   contexture=__version__,
                    )
         self._headers['hostname'] = env['host']
+        self._throttled = 0
+        self._reconnect_wait = reconnect_wait
 
-        self.thread = threading.Thread(target=self.run)
-        self.thread.daemon = True
-        self.thread.start()
+        # Rely on attributes resolving up to the class level
+        # for singleton behavior.
+        if singleton:
+            target = self.__class__
+        else:
+            target = self
+        if not target._queue or not singleton:
+            target._queue = Queue(maxqueue)
+            target._thread = threading.Thread(target=self.run)
+        # if not target._thread.is_active():
+            target._thread.daemon = True
+            target._thread.start()
 
-        self.emit(faux_record(env))
+        self.emit_obj(env)
 
         logging.Handler.__init__(self)
 
@@ -58,7 +87,7 @@ class AMQPHandler(logging.Handler):
         return pika.SelectConnection(pika.URLParameters(self._url),
                                      self.on_connection_open)
 
-    def on_connection_closed(self, method_frame, *args):
+    def on_connection_closed(self, method_frame):
         """This method is invoked by pika when the connection to RabbitMQ is
         closed unexpectedly. Since it is unexpected, we will reconnect to
         RabbitMQ if it disconnects.
@@ -183,30 +212,36 @@ class AMQPHandler(logging.Handler):
                 self._running = True
                 self._connection = self.connect()
                 self._connection.ioloop.start()
-            except Exception:
+            except Exception, e:
+                LOGGER.info('Sleeping for %s seconds and retrying'
+                            % self._reconnect_wait)
+                LOGGER.exception(e)
                 self._running = False
-                # Free up queued objects
-                # with self._queue.mutex:
-                #     self._queue.queue.clear()
-                LOGGER.info('Sleeping for 10 seconds and retrying')
-                time.sleep(10)
+                time.sleep(self._reconnect_wait)
 
     def close_channel(self):
         """Invoke this command to close the channel with RabbitMQ by sending
         the Channel.Close RPC command.
 
         """
-        LOGGER.info('Closing the channel')
-        self._channel.close()
+        if self._channel:
+            LOGGER.info('Closing the channel')
+            self._channel.close()
+        else:
+            LOGGER.debug('Channel has not been open')
 
     def close_connection(self):
         """This method closes the connection to RabbitMQ."""
-        LOGGER.info('Closing connection')
-        self._connection.close()
+        if self._connection:
+            LOGGER.info('Closing connection')
+            self._connection.close()
+        else:
+            LOGGER.debug('Connection has not been open')
 
     def stop(self):
         """
-        stop() from example. We'll be exiting uncleanly, so messages might be lost.
+        stop() from example. We'll be exiting uncleanly, so messages might be
+        lost.
 
         Original docstring:
         Stop the example by closing the channel and connection. We
@@ -218,29 +253,37 @@ class AMQPHandler(logging.Handler):
 
         """
         # Stopping from __del__ quite work, but we don't care so much.
-        self.emit(faux_record({"stopping": True}))
+        self.emit_obj({"stopping": True})
         self._running = False
-        self.close_channel()
-        self.close_connection()
-        self._connection.ioloop.start()
+        if self._channel:
+            self.close_channel()
+        if self._connection:
+            self.close_connection()
+            self._connection.ioloop.start()
 
     def emit(self, record):
         try:
             self._queue.put_nowait(record)
         except Full:
-            LOGGER.warning('Queue full, discarding. Used %sK',
-                           resource.getrusage(resource.RUSAGE_SELF).ru_maxrss)
+            if self._throttled == 0:
+                size = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
+                LOGGER.warning('Queue full, discarding. Used %sK', size)
+            self._throttled += 1
 
-    # def filter(self, record):
-    #     return int(self._running)
+    def emit_obj(self, obj):
+        self.emit(faux_record(obj))
+
+    def publish_items(self):
+        for item in qitems(self._queue):
+            self.publish_record(item)
+        if self._throttled > 0:
+            LOGGER.warning('Queue overflow recovered, %s messages lost'
+                           % self._throttled)
+            self._throttled = 0
+            self.publish_record(faux_record({'recovered': self._throttled}))
 
     def schedule_next_message(self):
-        while True:
-            try:
-                item = self._queue.get_nowait()
-            except Empty:
-                break
-            self.publish_record(item)
+        self.publish_items()
         # if self._running:
         self._connection.add_timeout(self.INTERVAL, self.schedule_next_message)
 
@@ -273,8 +316,11 @@ class AMQPHandler(logging.Handler):
         except Exception, e:
             message = json.dumps(dict(error=repr(e)))
 
-        self._channel.basic_publish(exchange, destination,
-                                    message, properties)
+        if self._channel.is_open:
+            self._channel.basic_publish(exchange, destination,
+                                        message, properties)
+        else:
+            LOGGER.debug('Discarding %s', message)
 
     # Well, attempt to exit cleanly.
     def __del__(self):
