@@ -76,15 +76,18 @@ class Context(object):
         setattr(LogProxy, level_name, closure(level, level_name))
 
     def __init__(self,
+                 # Common configuration
                  name=None,         # amqp routing key
                  routing_key=None,  # leave empty to use name
                  headers={},        # amqp headers
                  context={},        # initial context
-                 obj=None,          # native object to wrap
-                 guid=None,         # provide your own ID
                  ignore=[],         # exclude from handling
-                 silent=False,      # for using LC from SH
                  logger=None,       # custom logger
+                 guid=None,         # provide your own ID
+
+                 # Behavior
+                 obj=None,          # native object to wrap
+                 silent=False,      # for using LC from SH
                  transient=False,   # transient object
                  ):
         self.context = {}
@@ -142,14 +145,6 @@ class Context(object):
         status = transient and 'transient' or 'born'
         self._update(meta=dict(status=status), context=context)
 
-    @classmethod
-    def fromdict(cls, d, **kwargs):
-        pass
-
-    @classmethod
-    def fromobj(cls, obj, **kwargs):
-        pass
-
     def _update(self, msg=[], level=logging.DEBUG, context={}, meta={}):
         self.context.update(context)
         obj = {}
